# This file is part of Remote Support Desktop
# https://gitlab.das-netzwerkteam.de/RemoteWebApp/rwa.support.sessionservice
# Copyright 2020, 2021 Jonathan Weth <dev@jonathanweth.de>
# Copyright 2020 Daniel Teichmann <daniel.teichmann@das-netzwerkteam.de>
# Copyright 2020 Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
# SPDX-License-Identifier: GPL-2.0-or-later
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the
# Free Software Foundation, Inc.,
# 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

import os
import secrets
import subprocess  # noqa
from typing import Dict

import port_for

from .lock import TEMP_DIR_PATH


def save_password(pw: str) -> str:
    """Save password in x11vnc format in temporary directory."""
    filename = os.path.join(TEMP_DIR_PATH, f"{secrets.token_urlsafe(20)}.pw")
    os.makedirs(TEMP_DIR_PATH, exist_ok=True)
    p = subprocess.Popen(["x11vnc", "-storepasswd", f"{pw}", filename])  # noqa
    p.communicate()
    return filename


def run_vnc(pw_filename: str) -> Dict[str, Dict[str, int]]:
    """Run x11vnc and websockify with random, unique ports in background."""
    port = port_for.select_random()
    port_vnc = port_for.select_random()

    # Start VNC process
    p = subprocess.Popen(["x11vnc", "-forever", "-rfbauth", pw_filename, "-rfbport", f"{port_vnc}"])  # noqa

    # Start websockify
    p2 = subprocess.Popen(["websockify", str(port), f"127.0.0.1:{port_vnc}"])  # noqa

    return {
        "ws": {"pid": p2.pid, "port": port},
        "vnc": {"port": port_vnc, "pid": p.pid},
    }
